% This is an example script showing how to segment images using the
% local center-of-mass based approach, proposed in:
%
% I. Aganj, M. G. Harisinghani, R. Weissleder, and B. Fischl,
% "Unsupervised medical image segmentation based on the local center of mass",
% Scientific Reports, vol. 8, Article no. 13012, 2018.
% www.nature.com/articles/s41598-018-31333-5
%
% See also:   segCM

% Codes by Iman Aganj
% http://nmr.mgh.harvard.edu/~iman

%% Read the input image
% The input image for segmentation can be 2D or 3D. In this example, we use
% a 2D slice of the magnetic resonance image of a knee, which is included
% in Matlab.
I = double(dicomread('knee1.dcm'));
I = I - min(I(:));
I = I / max(I(:));

%% Run the segmentation
% We first set the parameters. See the help of segCM.m for more options.
clear param
param.alpha = 300; % A larger alpha results in more segmented regions.
param.randIterNum = 300; % A smaller number of random iterations results in more segmented regions.
param.maxIter = 500; % Maximum number of iterations.
param.showFigure = true; % On-the-fly visualization.
% param.indexGPU = 1; % Uncomment to gain speed using a GPU.

% We now segment the image 'I' and save the resulting labels in 'L'.
L = segCM(I, param);

%% Visualize the results
figure

% Show the original image.
subplot(1,2,1)
imagesc(I)
axis equal tight
title Image
colormap(gca, gray)

% Show the labels with edges and a random colormap.
subplot(1,2,2)
edgeL = L;
edgeL([diff(L,1,1); zeros(1,size(L,2))] | [diff(L,1,2) zeros(size(L,1),1)]) = 0;
imagesc(edgeL)
axis equal tight
title Labels
cMap = hsv(max(L(:)));
colormap(gca, [0 0 0; cMap(randperm(size(cMap,1)),:)])
